#Turntable API

A simple nodejs wrapper for the turntable API

## Installation
    npm install ttapi

Find your `AUTH`, `USERID` and `ROOMID` informations with [that bookmarklet](http://alaingilbert.github.com/Turntable-API/bookmarklet.html). 

## Examples

### Chat bot

This bot respond to anybody who write "/hello" on the chat.

```js
var Bot = require('ttapi');
var bot = new Bot(AUTH, USERID, ROOMID);

bot.on('speak', function (data) {
   // Respond to "/hello" command
   if (data.text.match(/^\/hello$/)) {
      bot.speak('Hey! How are you '+data.name+' ?');
   }
});
```

### Http Server

This bot create an http server and give his version number when we ask for "http://127.0.0.1:8080/version/" this page.

```js
var Bot = require('ttapi');
var bot = new Bot(AUTH, USERID, ROOMID);
bot.listen(8080, '127.0.0.1');

var myScriptVersion = '0.0.0';

bot.on('httpRequest', function (req, res) {
   var method = req.method;
   var url    = req.url;
   switch (url) {
      case '/version/':
         if (method == 'GET') {
            res.writeHead(200, { 'Content-Type': 'application/json' });
            res.end('{"version":"'+myScriptVersion+'"}');
         } else {
            res.writeHead(500);
            res.end();
         }
         break;
      default:
         res.writeHead(500);
         res.end();
         break;
   }
});
```

### TCP Server

This bot open a tcp server. That will allow you to easily communicate with the bot via a terminal.

```js
var Bot = require('ttapi');
var bot = new Bot(AUTH, USERID, ROOMID);
bot.tcpListen(8080, '127.0.0.1');

var myScriptVersion = 'V0.0.0';

bot.on('tcpConnect', function (socket) { });
bot.on('tcpMessage', function (socket, msg) {
   if (msg == 'version') {
      socket.write('>> '+myScriptVersion+'\n');
   }
});
bot.on('tcpEnd', function (socket) { });
```

You can communicate with the bot like this:

    nc 127.0.0.1 8080

And then type:

    version


### Simple

```js
var Bot    = require('ttapi');
var AUTH   = 'auth+live+xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx';
var USERID = 'xxxxxxxxxxxxxxxxxxxxxxxx';
var ROOMID = 'xxxxxxxxxxxxxxxxxxxxxxxx';

var bot = new Bot(AUTH, USERID);

bot.on('ready',        function (data) { bot.roomRegister(ROOMID); });
bot.on('roomChanged',  function (data) { console.log('The bot has changed room.', data); });

bot.on('speak',        function (data) { console.log('Someone has spoken', data); });
bot.on('update_votes', function (data) { console.log('Someone has voted',  data); });
bot.on('registered',   function (data) { console.log('Someone registered', data); });
```

# Debugging

```js
bot.debug = true;
```

That will print on the terminal all the data that you get and all the data that you send.


# Documentation


## Events

[Here are some examples of the data that you'll receive from those events.](https://github.com/alaingilbert/Turntable-API/tree/master/turntable_data)

### on('tcpConnect', function (socket) { })

Triggered when a socket open a connection.

### on('tcpMessage', function (socket, msg) { })

Triggered when the bot receive a tcp message.

### on('tcpEnd', function (socket) { })

Triggered when a socket close its connection.

### on('httpRequest', function (request, response) { })

Triggered when the bot receive an http request.

### on('roomChanged', function ([data](https://github.com/alaingilbert/Turntable-API/blob/master/turntable_data/room_infos.js)) { })

Triggered when the bot enter in a room.

### on('registered', function ([data](https://github.com/alaingilbert/Turntable-API/blob/master/turntable_data/registered.js)) { })

Triggered when a user enter the room.

### on('deregistered', function ([data](https://github.com/alaingilbert/Turntable-API/blob/master/turntable_data/deregistered.js)) { })

Triggered when a user leave the room.

### on('speak', function ([data](https://github.com/alaingilbert/Turntable-API/blob/master/turntable_data/speak.js)) { })

Triggered when a new message is send via the chat.

### on('endsong', function () { })

Triggered at the end of the song. (Just before the newsong/nosong event)

### on('newsong', function ([data](https://github.com/alaingilbert/Turntable-API/blob/master/turntable_data/newsong.js)) { })

Triggered when a new song start.

### on('nosong', function ([data](https://github.com/alaingilbert/Turntable-API/blob/master/turntable_data/nosong.js)) { })

Triggered when there is no song.

### on('update_votes', function ([data](https://github.com/alaingilbert/Turntable-API/blob/master/turntable_data/update_votes.js)) { })

Triggered when a user vote.

### on('booted_user', function (data) { })

Triggered when a user is booted.

### on('update_user', function (data) { })

Triggered when a user change his name/infos.

### on('add_dj', function ([data](https://github.com/alaingilbert/Turntable-API/blob/master/turntable_data/add_dj.js)) { })

Triggered when a user take a dj spot.

### on('rem_dj', function ([data](https://github.com/alaingilbert/Turntable-API/blob/master/turntable_data/rem_dj.js)) { })

Triggered when a user leave a dj spot.

### on('new_moderator', function ([data](https://github.com/alaingilbert/Turntable-API/blob/master/turntable_data/new_moderator.js)) { })

Triggered when a user is granted to moderator title.

### on('rem_moderator', function ([data](https://github.com/alaingilbert/Turntable-API/blob/master/turntable_data/rem_moderator.js)) { })

Triggered when a user loose his moderator title.

### on('snagged', function ([data](https://github.com/alaingilbert/Turntable-API/blob/master/turntable_data/snagged.js)) { })

Triggered when a user queues the currently playing song.


## Actions

### tcpListen ( port, address )

Start a tcp server.

### listen ( port, address )

Start a http server.

### roomNow ( [callback:fn] )

Get the turntable server time.

### listRooms ( skip=0:int [, callback:fn] )

Get 20 rooms.

### roomRegister ( roomId:string [, callback:fn] )

Register in a room.

### roomDeregister ( [callback:fn] )

Deregister from the current room.

### roomInfo ( [[extended=true:bool, ]callback:fn] )

Get the current room informations. Do not include song log if 'extended' is false.

### speak ( msg:string [, callback:fn] )

Broadcast a message on the chat.

### bootUser ( userId:string, reason:string [, callback:fn] )

Boot a user.

### addModerator ( userId:string [, callback:fn] )

Add a moderator.

### remModerator ( userId:string [, callback:fn] )

Remove a moderator.

### addDj ( [callback:fn] )

Add yourself as a Dj.

### remDj ( [[userId:string, ]callback:fn] )

Remove a Dj.

### stopSong ( [callback:fn] )

Skip the current song.

### vote ( val:enum('up', 'down') [, callback:fn] )

Vote for the current song.

### userAuthenticate ( [callback:fn] )

Authenticate the user.

### userInfo ( [callback:fn] )

Get the current user informations.

### getProfile ( [[userId:string, ]callback:fn] )

Get a user profile.

### modifyProfile ( profile:obj [, callback:fn] )

Modify your profile. Any missing properties from the 'profile' object will be replaced with the current values.

#### Arguments

* `profile`:obj (required)
  * `name`:string (optional)
  * `twitter`:string (optional)
  * `facebook`:string (optional)
  * `website`:string (optional)
  * `about`:string (optional)
  * `topartists`:string (optional)
  * `hangout`:string (optional)
* `callback`:fn (optional)

#### Examples

```js
bot.modifyProfile({ website:'http://ttdashboard.com/', about:'My bot.' }, callback);
```

### modifyLaptop ( laptop:enum('linux', 'mac', 'pc', 'chrome') [, callback:fn] )

Modify your laptop.

### modifyName ( name:string [, callback:fn] )

Modify your name.

### setAvatar ( avatarId:int [, callback:fn] )

Set your avatar.

### becomeFan ( userId:string [, callback:fn] )

Fan someone.

### removeFan ( userId:string [, callback:fn] )

Unfan someone.

### playlistAll ( [ playlistName:string, ] callback:fn )

Get all informations about a playlist.

#### Arguments

* `playlistName` (optional) default: `default`
* `callback` (required)

#### Examples

```js
bot.playlistAll(callback);
bot.playlistAll(playlistName, callback);
```

### playlistAdd ( [ playlistName:string, ] songId:string [, index:int [, callback:fn]] )

Add a song on a playlist.

#### Arguments

* `playlistName` (optional) default: `default`
* `songId` (required)
* `index` (optional) default: `0`
* `callback` (optional)

#### Examples

```js
bot.playlistAdd(songId);
bot.playlistAdd(songId, idx);
bot.playlistAdd(songId, callback);
bot.playlistAdd(songId, idx, callback);
bot.playlistAdd(playlistName, songId, idx);
bot.playlistAdd(playlistName, songId, callback);
bot.playlistAdd(playlistName, songId, idx, callback);
bot.playlistAdd(false, songId, callback); // Backward compatibility
bot.playlistAdd(false, songId);           // Backward compatibility
````

### playlistRemove ( [ playlistName:string, ] index:int [, callback:fn ] )

Remove a song on a playlist.

#### Arguments

* `playlistName` (optional) default: `default`
* `index` (optional) default: `0`
* `callback` (optional)

#### Examples
```js
bot.playlistRemove();
bot.playlistRemove(index);
bot.playlistRemove(index, callback);
bot.playlistRemove(playlistName, index);
bot.playlistRemove(playlistName, index, callback);
```

### playlistReorder ( [ playlistName:string, ] indexFrom:int, indexTo:int [, callback:fn ] )

Reorder a playlist. Take the song at index `indexFrom` and move it to index `indexTo`.

#### Arguments

* `playlistName` (optional) default: `default`
* `indexFrom` (required) default: `0`
* `indexTo` (required) default: `0`
* `callback` (optional)

#### Examples

```js
bot.playlistReorder(indexFrom, indexTo);
bot.playlistReorder(indexFrom, indexTo, callback);
bot.playlistReorder(playlistName, indexFrom, indexTo);
bot.playlistReorder(playlistName, indexFrom, indexTo, callback);
```
